//
//
// C++ Implementation for module: Pattern_Miner
//
// Description: 
//
//
// Author: exa
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "Pattern_Miner.hxx"

#include "fp-tree/FP_Tree.hxx"
#include "transaction-db/Transaction_Set.hxx"
//#include "2-items/Count.hxx"
//#include "parallel/Parallel.hxx"

Pattern_Miner::Pattern_Miner(const string& name,
			     Transaction::File_Format format,
			     bool dbg)
  : db_name(name), my_ts(name, format), ts(my_ts),
    relative_support(false),
    counter(0), given_counter(false), debug(dbg)
{
}

Pattern_Miner::Pattern_Miner(Transaction_Set &_ts, bool dbg)
  : ts(_ts), relative_support(false),
    counter(0), given_counter(false), debug(dbg)
{
}

Pattern_Miner::Pattern_Miner(Transaction_Set &_ts, TS_Counter &ctr,
			     bool dbg)
  : ts(_ts), relative_support(false),
    counter(&ctr), given_counter(true), debug(dbg)
{
}

Pattern_Miner::~Pattern_Miner() {
}

void Pattern_Miner::discover_relative(float sup, const string &out)
{
  real_support = sup;
  relative_support = true;

  discover(out);
}

void Pattern_Miner::discover_absolute(int sup, const string &out)
{
  // determine count and frequent items
  support = sup;
  relative_support = false;

  discover(out);
}

void Pattern_Miner::discover(const string &out)
{
  if (out != "") {
    write_output = true;
    pattern_filename = out;
  }
  else write_output = false;

  map<int,int> pat_counts;	// length -> cardinality map for patterns

  TS_Counter* counter;
  if (!given_counter) {
    counter = new TS_Counter(ts);
  }
  else
    counter = this->counter;

  // determine count and frequent items
  cerr << "[pass 1] counting large items in " << db_name << endl;
  CPU_Time t_s;
  if (!given_counter) {
    // count 1 items
    if (relative_support)
      counter->count_items_relative(real_support);
    else
      counter->count_items_absolute(support);
  }
  cerr << "|F|=" << counter->F.size() << " ";
  if (debug) {
    cout << endl << "F = " << counter->F
	 << " count = " << counter->count << endl;
  }
  CPU_Time t_f;
  cerr << "[" << t_f - t_s << " sec.] " << endl;

  // construct fp tree
  cerr << "[pass 2] constructing fp-tree ";
  FP_Tree fp_tree(counter->F, counter->count, pat_counts, debug);
  Transaction_Set::Scanner scanner(ts);
  while (!scanner.end()) {
    Transaction t_i;
    if (scanner.read(t_i)) {
      fp_tree.insert(t_i);
    }
  }
  CPU_Time t_d;
  cerr << "[" << t_d - t_f << " sec.] " << endl;

  // discover patterns
  cerr << "discovering frequent patterns ";
  if (write_output) {
    ofstream pattern_out(pattern_filename.c_str());
    pattern_out << "(" << counter->num_transactions << ")" << endl;
    fp_tree.discover_patterns(counter->support, &pattern_out);
  }
  else {
    fp_tree.discover_patterns(counter->support);
  }
  CPU_Time t_e;
  cerr << "[" << t_e - t_d << " sec.] " << endl;
  cerr << "end run [" << t_e - t_s << " sec.] " << endl;
  if (write_output)
    fp_tree.print_pattern_counts(cout);

  if (!given_counter)
    delete counter;
}
