//
//
// C++ Implementation for module: Transaction_Set
//
// Description: 
//
//
// Author: exa
//
// Copyright: See COPYING file that came with this distribution
//
//

#include "Transaction_Set.hxx"

using namespace Utility;

Transaction_Set::Transaction_Set()
  : external(false)
{
}

Transaction_Set::Transaction_Set(const string& fn,
				 Transaction::File_Format _format)
  : filename(fn), format(_format), external(true)
{
}

// scanner class does some real work
// simplistic implementation without caching

Transaction_Set::Scanner::Scanner(Transaction_Set& _ts)
  : ts(_ts), file_error(false)
{
  if (ts.is_external()) {
    if (ts.format==Transaction::binary)
      in.open(ts.filename.c_str(), ios::binary);
    else if (ts.format==Transaction::ascii)
      in.open(ts.filename.c_str());
    if (!in) {
      cerr << "error opening file " << ts.filename << endl;
      exit(-1);
    }
  }
  else {
    it = ts.begin();
  }
//     throw string("File Error!");
}

// scan next transaction
bool Transaction_Set::Scanner::read(Transaction& t_i)
{
  if (ts.is_external()) {
    //bool a = ts.ascii;
    if (t_i.read(in, ts.format)) {
//       ts.push_back(t_i);		// cache transaction
      return true;
    }
    else {
      if (!in.eof())
	cerr << endl
	     << "FILE ERROR: end of file not met after a null transaction"
	     << endl;
      file_error=true;
      return false;
    }
  }
  else {
    t_i = *it;
    it++;
    return true;
  }
}

bool Transaction_Set::Scanner::end()
{
  if (ts.is_external()) {
    bool end = in.eof() || file_error;
//     if (end)
//       ts.external=false;
    return end;
  }
  else
    return it == ts.end();
}

void Transaction_Set::encode(vector<int>& msg)
{
  for (iterator i=begin(); i!=end(); i++) {
    Transaction &t_i = *i;
    t_i.encode_step(msg);
  }
}

void Transaction_Set::decode(vector<int>& msg)
{
  if (msg.size()==0)
    return;
  vector<int>::iterator msg_it = msg.begin();
  while (msg_it < msg.end()) {
    Transaction& t_i = new_transaction();
    msg_it = t_i.decode_step(msg_it);
  }
}

void Transaction_Set::read(istream& in)
{
  while (!in.eof()) {
    iterator i = insert(end(), Transaction());
    if (!i->read(in, format))
      erase(i);
  }
//   cout << "Transaction_Set: read " << size()
//        << " transactions" << endl;
}

void Transaction_Set::write(ostream& out)
{
  for (iterator i=begin(); i!=end(); i++) {
    i->write(out);
  }
//   cout << "Transaction_Set: wrote " << size() << " transactions" << endl;
}

class Less_Than_E
{
public:
  Less_Than_E(int _e): e(_e) {}
  Less_Than_E() {}
  bool operator()(pair<const int, int> x) { return x.second<e; }
  int e;
};

void Transaction_Set::large_items(set<int>& L, int epsilon) const
{
  typedef map<int,int> freq_map;
  freq_map freq;
  for (const_iterator i=begin(); i!=end(); i++) {
    const Transaction& t_i = *i;
//     cout << t_i << endl;
    for (Transaction::const_iterator itemi=t_i.begin();
	 itemi!=t_i.end();itemi++) {
      int item = *itemi;
      if (freq.find(item)!=freq.end())
	freq[item] = freq[item] + 1;
      else
	freq[item] = 0;
    }
  }
  //cout << "freq=" << freq << endl;
  for (map<int,int>::iterator j=freq.begin(); j!=freq.end(); j++) {
    if (j->second>=epsilon)
      L.insert(j->first);
  }
    //Less_Than_E less_than(epsilon);
  //remove_if(freq.begin(), freq.end(), less_than);
}

void Transaction_Set::prune_not_in(set<int>& F)
{
  for (iterator i=begin(); i!=end(); i++) {
    Transaction& t_i = *i;
//     t_i.make_set(); // maintain as set
    t_i.prune_not_in(F);
  }
}

void Transaction_Set::prune_small_items(int epsilon)
{
  set<int> L;
  large_items(L, epsilon);
  //cout << "L=" << L << endl;
//   cout << L.size() << " large items" << endl;
  prune_not_in(L);
}

void Transaction_Set::stats()
{
  int freq=0;
  for (const_iterator i=begin(); i!=end(); i++) {
    const Transaction& t_i = *i;
    freq += t_i.size();
  }
  cout << "total number of items " << freq << endl;
}

void Transaction_Set::split(istream& in, int k, const string& outprefix)
{
  int db = 0;
  vector<ofstream*> out(k); // a vector of k ofstream pointers
  for (int i=0; i<k; i++) { // open a file for each part
    string fn;
    ostringstream os(fn);
    os << outprefix << '.' << i;
    out[i] = new ofstream(os.str().c_str(), ios::binary);
  }
  while (!in.eof()) {
    Transaction t_i;
    if (t_i.read(in, Transaction::binary)) {
      // write it in db
     t_i.write(*out[db]);
#ifndef NDEBUG
     //cout << "wrote txn " << t_i << " to db " << db << endl;
#endif
    }
    db = (db + 1) % k;
  }
  for (int i=0; i<k; i++) {
    ofstream* o = out[i];
    delete o;
  }
  cout << "split database into " << k << endl;
}

void Transaction_Set::clear()
{
  erase(begin(), end());
}

