//
//
// C++ Interface for module: Transaction_Set
//
// Description:
//
//
// Author: exa
//
// Copyright: See COPYING file that came with this distribution
//
//

#ifndef Transaction_Set_Interface
#define Transaction_Set_Interface

#include "generic/General.hxx"
#include "Transaction.hxx"

// implements as a vector of transactions but that is going to change
class Transaction_Set
: public list<Transaction>
{
public:
  typedef list<Transaction> Base;

  Transaction_Set();
  Transaction_Set(const string& fn, Transaction::File_Format format);

  class Scanner {
  public:
    Scanner(Transaction_Set& ts);
    bool read(Transaction& t_i);
    bool end();
    Transaction_Set& ts;
    ifstream in;
    bool file_error;
    Transaction_Set::iterator it;
  };
  friend class Scanner;

  void add(const Transaction& t_i) {
    push_back(t_i);
  }

  Transaction& new_transaction() {
    add(Transaction());
    Transaction& t_i = back();
    return t_i;
  }

  void clear();

  // vector's contents is the message for now
  void encode(vector<int>& msg);
  void decode(vector<int>& msg);

  bool is_external() { return external; }
  const string& get_filename() { return filename; }

private:
  string filename;
  Transaction::File_Format format;
  bool external; // run in out-of-core mode?

// static routines

public:
  // count 1 items in a transaction db
  // int count_items(vector<int>& count);
  static int count_items(istream& in, vector<int>& count);

  //   static void count_2_items(istream& in, Graph& G, const set<int>& L);

  // split randomly a transaction set
  static void split(istream& in, int k, const string & outprefix );

// old routines

  // this surely doesn't fit in the memory
  void read(istream& in);
  void write(ostream& out);
  void large_items(set<int>& L, int epsilon) const;
  void prune_not_in(set<int>& items);
  void prune_small_items(int);
  void stats();
};

#endif
